﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Globalization;
using System.IO.Ports;
using System.Linq;
using System.Threading;
using System.Windows;
using System.Windows.Data;

namespace InterLinker
{
    public class ComPortSideConverter : IValueConverter
    {
        object IValueConverter.Convert(object value, Type targetType, object parameter, CultureInfo culture)
        {
            return ((string)value).Split('-')[0];
        }

        object IValueConverter.ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }

    public class ConversationLine
    {
        public string Port { get; set; }
        public string Side { get; set; }
        public List<byte> Data { get; set; }
        public string DataAsString { get { return "0x" + BitConverter.ToString(Data.ToArray()).Replace("-", " 0x"); } }
    }

    public partial class MainWindow : Window
    {
        public List<ConversationLine> ConversationData { get; set; }
        private static SerialPort port1, port2;
        public MainWindow()
        {
            InitializeComponent();
            ConversationData = new List<ConversationLine>();

            BackgroundWorker bg = new BackgroundWorker();
            bg.DoWork += Bg_DoWork;
            bg.RunWorkerAsync();
        }

        private void Bg_DoWork(object sender, DoWorkEventArgs e)
        {
            while (true)
            {
                Dispatcher.BeginInvoke(((Action)(() =>
                {
                    var l = ConversationData.ToList();
                    FromText.ItemsSource = null;
                    FromText.ItemsSource = l;
                })));
                Thread.Sleep(1500);
            }
        }

        private string lastPort;
        ConversationLine currentLine = null;

        private void btnCopy_Click(object sender, RoutedEventArgs e)
        {
            Clipboard.SetText(string.Join("\r\n", ConversationData.Select(x => "<div class='" + x.Side + " " + x.Port + "'>" + x.DataAsString + "</div>")));
        }

        private void btnStart_Click(object sender, RoutedEventArgs e)
        {
            port1 = new SerialPort(txtPort1.Text, 9600, Parity.None, 8, StopBits.One);
            port1.DataReceived += new SerialDataReceivedEventHandler(Port_DataReceived);
            port2 = new SerialPort(txtPort2.Text, 9600, Parity.None, 8, StopBits.One);
            port2.DataReceived += new SerialDataReceivedEventHandler(Port_DataReceived);
            //do this last, incase a message comes straight away.
            port1.Open();
            port2.Open();
        }

        private void Port_DataReceived(object sender, SerialDataReceivedEventArgs e)
        {
            var port = ((SerialPort)sender);
            int btr = port.BytesToRead;
            byte[] data = new byte[btr];
            for (int b = 0; b < btr; b++)
            {
                int fromPort = port.ReadByte();
                data[b] = (byte)fromPort;
            }

            if (lastPort != port.PortName)
            {
                if (currentLine != null)
                {
                    ConversationData.Add(currentLine);
                }
                currentLine = new ConversationLine()
                {
                    Port = port.PortName,
                    Side = port.PortName == port1.PortName ? "Left" : "Right",
                    Data = data.ToList(),
                };
            }
            else
            {
                currentLine.Data.AddRange(data);
            }
            if (sender == port1)
            {
                lastPort = port.PortName;
                port2.Write(data, 0, data.Count());
            }
            else
            {
                lastPort = port.PortName;
                port1.Write(data, 0, data.Count());
            }
        }
    }
}
